def load_data(filename):
    """Prende un ingresso un file di interazioni elemento regolatore - UTR.
Restituisce un dizionario con elemento regolatore come chiave e elenco dei suoi target come valore."""
    f = open(filename)
    # rimuove intestazione
    f.readline()
    d = {}
    for row in f:
        data = row.split()
        if data[0] in d:
            d[data[0]].add(data[1])
        else:
            d[data[0]] = set([data[1]])
    return d

def extract_frequent_regulators(diz, thresh):
    """Prende in ingresso un dizionario elemento regolatore - insieme di target 
e una soglia. Restituisce la lista di elementi regolatori che hanno un numero di 
interazioni superiore alla soglia."""
    # regulators = []
    # for (k,v) in diz.items():
    #     if len(v) > thresh:
    #         regulators.append(k)
    # return regulators    
    return [k for (k,v) in diz.items() if len(v) > thresh]

def compute_shared_interactions(diz, regulators):
    """Prende in ingresso un dizionario elemento regolatore - insieme di target 
e una lista di elementi regolatori. Restituisce una matrice con il numero di interazioni 
in comune tra ciascuna coppia di elementi regolatori."""
    int_matrix = {}
    for r1 in regulators:
        for r2 in regulators:
            # calcolo numero di interazioni in comune
            if r1 == r2:
                int_matrix[(r1,r1)] = len(diz[r1])
            else:
                interactions = len(diz[r1].intersection(diz[r2]))
                int_matrix[(r1,r2)] = interactions
    return int_matrix

def print_shared_interactions(int_matrix, regulators):
    """Prende in ingresso un dizionario coppia di elementi regolatori - numero 
di interazioni in comune e una lista di elementi regolatori e stampa il dizionario in forma matriciale."""
    # stampa intestazione
    print("reg\t" + "\t".join(regulators))
    for r1 in regulators:
        # costruisce la riga corrispondente a r1
        # AGO2  21711  17867  14358  15654  13133
        buf=r1
        for r2 in regulators:
            buf += "\t" + str(int_matrix[(r1,r2)])
        print(buf)


filename = input("Inserire nome file: ")
thres = int(input("Inserire soglia: "))

d = load_data(filename)
l = extract_frequent_regulators(d, thres)
shared = compute_shared_interactions(d, l)
print_shared_interactions(shared, l)
